@@ -5,109 +5,54 @@
 #  joshua@mulliken.net to receive a copy
 import datetime
 import hashlib
+import json
+import logging
 import time
-import uuid
-from enum import Enum
-from typing import List, Dict
+from typing import Any, Dict, Optional
 
 import requests
 
-PHONE_SYSTEM_TYPE = "1"
-API_KEY = "WMXHYf79Nr5gIlt3r0r7p9Tcw5bvs6BB4U8O8nGJ"
-APP_VERSION = "2.18.43"
-APP_VER = "com.hualai.WyzeCam___2.18.43"
-APP_NAME = "com.hualai.WyzeCam"
-PHONE_ID = str(uuid.uuid4())
+from .const import (
+    API_KEY,
+    APP_VERSION,
+    PHONE_SYSTEM_TYPE,
+    APP_VER,
+    PHONE_ID,
+    APP_NAME,
+    OLIVE_APP_ID,
+    APP_INFO
+)
+from .crypto import olive_create_signature
+from .exceptions import (
+    ParameterError,
+    AccessTokenError,
+    UnknownApiError,
+    ActionNotSupported
+)
+from .payload_factory import (
+    ford_create_payload,
+    olive_create_get_payload,
+    olive_create_post_payload,
+    olive_create_hms_payload,
+    olive_create_hms_patch_payload,
+    olive_create_hms_get_payload
+)
+from .types import ResponseCodes, Device, DeviceTypes, ThermostatProps, Group
+
+_LOGGER = logging.getLogger(__name__)
 
 
-class DeviceTypes(Enum):
-    LIGHT = "Light"
-    PLUG = "Plug"
-    OUTDOOR_PLUG = "OutdoorPlug"
-    MESH_LIGHT = "MeshLight"
-    CAMERA = "Camera"
-    CHIME_SENSOR = "ChimeSensor"
-    CONTACT_SENSOR = "ContactSensor"
-    MOTION_SENSOR = "MotionSensor"
-    WRIST = "Wrist"
-    BASE_STATION = "BaseStation"
-    SCALE = "WyzeScale"
-    LOCK = "Lock"
-    GATEWAY = "gateway"
-    COMMON = "Common"
-    VACUUM = "JA_RO2"
-    HEADPHONES = "JA.SC"
-    THERMOSTAT = "Thermostat"
-    GATEWAY_V2 = "GateWay"
-
-
-class PropertyIDs(Enum):
-    ON = "P3"
-    AVAILABLE = "P5"
-    BRIGHTNESS = "P1501"  # From 0-100
-    COLOR_TEMP = "P1502"  # In Kelvin
-    COLOR = "P1507"  # As a hex string RrGgBb
-    DOOR_OPEN = "P2001"  # 0 if the door is closed
-
-
-class ResponseCodes(Enum):
-    SUCCESS = "1"
-    PARAMETER_ERROR = "1001"
-    ACCESS_TOKEN_ERROR = "2001"
-
-
-class ResponseCodesLock(Enum):
-    SUCCESS = 0
-
-
-SWITCHABLE_DEVICES = [DeviceTypes.LIGHT, DeviceTypes.MESH_LIGHT, DeviceTypes.PLUG]
-
-
-class ActionNotSupported(Exception):
-    def __init__(self, device_type):
-        message = "The action specified is not supported by device type: {}".format(device_type)
-
-        super().__init__(message)
-
-
-class ParameterError(Exception):
-    pass
-
-
-class AccessTokenError(Exception):
-    pass
-
-class LoginError(Exception):
-    pass
-
-
-class UnknownApiError(Exception):
-    def __init__(self, response_json):
-        super(UnknownApiError, self).__init__(str(response_json))
-
-
-class Device:
-    product_type: str
-    product_model: str
-    mac: str
-    nickname: str
-
-    def __init__(self, dictionary):
-        for k, v in dictionary.items():
-            setattr(self, k, v)
-
-    def __repr__(self):
-        return "<Device: {}, {}>".format(DeviceTypes(self.product_type), self.mac)
-
-
-class BaseClient:
+class NetClient:
     access_token = ""
     refresh_token = ""
 
-    def login(self, email, password) -> bool:
-        email = email
-        password = password
+    def __init__(self):
+        self._session = requests.Session()
 
+    def __del__(self):
+        self._session.close()
+
+    def login(self, email, password) -> bool:
         login_payload = {
             "email": email,
             "password": self.create_password(password)
@@ -117,20 +62,18 @@ class BaseClient:
             "X-API-Key": API_KEY
         }
 
-        response_json = requests.post("https://auth-prod.api.wyze.com/user/login",
-                                      headers=headers, json=login_payload).json()
+        response_json: dict = self._session.post("https://auth-prod.api.wyze.com/user/login",
+                                                 headers=headers, json=login_payload).json()
 
-        # response_json = requests.post("https://auth-prod.api.wyze.com/user/login",
-        #                               json=login_payload).json()
-
-        try:
-            self.access_token = response_json['access_token']
-            self.refresh_token = response_json['refresh_token']
-            return True
-        except KeyError as e:
-            print(e)
+        if response_json.get('errorCode') is not None:
+            _LOGGER.error(f"Unable to login with response from Wyze: {response_json}")
             return False
 
+        self.access_token = response_json['access_token']
+        self.refresh_token = response_json['refresh_token']
+
+        return True
+
     def can_login(self, username, password):
         return self.login(username, password)
 
@@ -160,41 +103,6 @@ class BaseClient:
             else:
                 raise UnknownApiError(response_json)
 
-    app_key = "275965684684dbdaf29a0ed9"
-    app_secret = "4deekof1ba311c5c33a9cb8e12787e8c"
-
-    def postMethod(self, str, str2, map):
-        str4 = str
-        str3 = str2
-        map2 = map
-
-        return self.getTransferParam(map2, str3, str4, "post")
-
-    def getTransferParam(self, map, str4, str2, str3):
-        map["accessToken"] = self.access_token
-        map["key"] = self.app_key
-        map["timestamp"] = str(datetime.datetime.now().timestamp()).split(".")[0] + "000"
-        map["sign"] = self.getSignStr(str4, str3, map)
-        import json
-        print(json.dumps(map))
-        return map
-
-    def getSignStr(self, str, str2, map: Dict):
-        string_buf = str2 + str
-        for entry in sorted(map.keys()):
-            print(entry)
-            print(map[entry])
-            string_buf += entry + "=" + map[entry] + "&"
-
-        string_buf = string_buf[:-1]
-        print(string_buf)
-        string_buf += self.app_secret
-        import urllib.parse
-        urlencoded = urllib.parse.quote_plus(string_buf)
-        sign_str = hashlib.md5(urlencoded.encode()).hexdigest()
-        print(sign_str)
-        return sign_str
-
     def get_object_list(self):
         payload = {
             "phone_system_type": PHONE_SYSTEM_TYPE,
@@ -208,10 +116,9 @@ class BaseClient:
             "app_name": APP_NAME
         }
 
-        response_json = requests.post("https://api.wyzecam.com/app/v2/home_page/get_object_list",
-                                      json=payload).json()
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/home_page/get_object_list",
+                                           json=payload).json()
 
-        print(payload)
         self.check_for_errors(response_json)
 
         return response_json
@@ -232,8 +139,50 @@ class BaseClient:
             "target_pid_list": []
         }
 
-        response_json = requests.post("https://api.wyzecam.com/app/v2/device/get_property_list",
-                                      json=payload).json()
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/device/get_property_list",
+                                           json=payload).json()
+
+        self.check_for_errors(response_json)
+
+        return response_json
+
+    def get_auto_group_list(self):
+        payload = {
+            "access_token": self.access_token,
+            "app_name": APP_NAME,
+            "app_ver": APP_VER,
+            "app_version": APP_VERSION,
+            "group_type": "0",
+            "phone_id": PHONE_ID,
+            "phone_system_type": PHONE_SYSTEM_TYPE,
+            "sc": "9f275790cab94a72bd206c8876429f3c",
+            "sv": "9d74946e652647e9b6c9d59326aef104",
+            "ts": int(time.time()),
+        }
+
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/auto_group/get_list",
+                                           json=payload).json()
+
+        self.check_for_errors(response_json)
+
+        return response_json
+
+    def get_device_info(self, device: Device):
+        payload = {
+            "phone_system_type": PHONE_SYSTEM_TYPE,
+            "app_version": APP_VERSION,
+            "app_ver": APP_VER,
+            "device_mac": device.mac,
+            "sc": "9f275790cab94a72bd206c8876429f3c",
+            "ts": int(time.time()),
+            "device_model": device.product_model,
+            "sv": "c86fa16fc99d4d6580f82ef3b942e586",
+            "access_token": self.access_token,
+            "phone_id": PHONE_ID,
+            "app_name": APP_NAME
+        }
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/device/get_device_Info",
+                                           json=payload).json()
 
         self.check_for_errors(response_json)
 
@@ -272,10 +221,61 @@ class BaseClient:
             ]
         }
 
-        response_json = requests.post("https://api.wyzecam.com/app/v2/auto/run_action_list", json=payload).json()
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/auto/run_action_list", json=payload).json()
 
         self.check_for_errors(response_json)
 
+        return response_json
+
+    def auto_group_run(self, group: Group) -> dict:
+        payload = {
+            "access_token": self.access_token,
+            "app_name": APP_NAME,
+            "app_ver": APP_VER,
+            "app_version": APP_VERSION,
+            "group_id": group.group_id,
+            "phone_id": PHONE_ID,
+            "phone_system_type": PHONE_SYSTEM_TYPE,
+            "sc": "9f275790cab94a72bd206c8876429f3c",
+            "sv": "9d74946e652647e9b6c9d59326aef104",
+            "ts": int(time.time()),
+        }
+
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/auto_group/run", json=payload).json()
+
+        self.check_for_errors(response_json)
+
+        return response_json
+
+    def run_action(self, device: Device, action: str) -> dict:
+        if DeviceTypes(device.product_type) not in [
+            DeviceTypes.CAMERA
+        ]:
+            raise ActionNotSupported(device.product_type)
+
+        payload = {
+            "phone_system_type": PHONE_SYSTEM_TYPE,
+            "app_version": APP_VERSION,
+            "app_ver": APP_VER,
+            "sc": "9f275790cab94a72bd206c8876429f3c",
+            "ts": int(time.time()),
+            "sv": "9d74946e652647e9b6c9d59326aef104",
+            "access_token": self.access_token,
+            "phone_id": PHONE_ID,
+            "app_name": APP_NAME,
+            "provider_key": device.product_model,
+            "instance_id": device.mac,
+            "action_key": action,
+            "action_params": {},
+            "custom_string": "",
+        }
+
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/auto/run_action", json=payload).json()
+
+        self.check_for_errors(response_json)
+
+        return response_json
+
     def set_property_list(self, device: Device, plist):
         if DeviceTypes(device.product_type) not in [
             DeviceTypes.LIGHT
@@ -296,10 +296,13 @@ class BaseClient:
             "device_model": device.product_model,
             "device_mac": device.mac
         }
-        response_json = requests.post("https://api.wyzecam.com/app/v2/device/set_property_list", json=payload).json()
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/device/set_property_list",
+                                           json=payload).json()
 
         self.check_for_errors(response_json)
 
+        return response_json
+
     def set_property(self, device: Device, pid, pvalue):
         """
         Sets a single property on the selected device.
@@ -332,10 +335,12 @@ class BaseClient:
             "device_model": device.product_model,
             "device_mac": device.mac
         }
-        response_json = requests.post("https://api.wyzecam.com/app/v2/device/set_property", json=payload).json()
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/device/set_property", json=payload).json()
 
         self.check_for_errors(response_json)
 
+        return response_json
+
     def get_event_list(self, device: Device, count: int) -> dict:
         """
         Gets motion events from the event listing endpoint.
@@ -372,26 +377,157 @@ class BaseClient:
             "access_token": self.access_token
         }
 
-        response_json = requests.post("https://api.wyzecam.com/app/v2/device/get_event_list", json=payload).json()
+        response_json = self._session.post("https://api.wyzecam.com/app/v2/device/get_event_list", json=payload).json()
 
         self.check_for_errors(response_json)
 
         return response_json
 
     def lock_control(self, device: Device, action: str):
-        sb2 = "https://yd-saas-toc.wyzecam.com/openapi/lock/v1/control"
-        str3 = "/openapi/lock/v1/control"
+        url_path = "/openapi/lock/v1/control"
 
-        uuid = device.mac.split(".")[2]
+        device_uuid = device.mac.split(".")[2]
 
-        hash_map = {
-            "uuid": uuid,
+        payload = {
+            "uuid": device_uuid,
             "action": action  # "remoteLock" or "remoteUnlock"
         }
-
-        payload = self.postMethod(sb2, str3, hash_map)
+        payload = ford_create_payload(self.access_token, payload, url_path, "post")
 
         url = "https://yd-saas-toc.wyzecam.com/openapi/lock/v1/control"
 
-        response_json = requests.post(url, json=payload).json()
-        print(response_json)
+        response_json = self._session.post(url, json=payload).json()
+
+        return response_json
+
+    def thermostat_get_iot_prop(self, device: Device):
+        payload = olive_create_get_payload(device.mac)
+        signature = olive_create_signature(payload, self.access_token)
+        headers = {
+            'Accept-Encoding': 'gzip',
+            'User-Agent': 'myapp',
+            'appid': OLIVE_APP_ID,
+            'appinfo': APP_INFO,
+            'phoneid': PHONE_ID,
+            'access_token': self.access_token,
+            'signature2': signature
+        }
+
+        url = 'https://wyze-earth-service.wyzecam.com/plugin/earth/get_iot_prop'
+        response_json = self._session.get(url, headers=headers, params=payload).json()
+
+        self.check_for_errors_thermostat(response_json)
+
+        return response_json
+
+    def thermostat_set_iot_prop(self, device: Device, prop: ThermostatProps, value: Any):
+        url = "https://wyze-earth-service.wyzecam.com/plugin/earth/set_iot_prop_by_topic"
+        payload = olive_create_post_payload(device.mac, device.product_model, prop, value)
+        signature = olive_create_signature(json.dumps(payload, separators=(',', ':')), self.access_token)
+        headers = {
+            'Accept-Encoding': 'gzip',
+            'User-Agent': 'myapp',
+            'appid': OLIVE_APP_ID,
+            'appinfo': APP_INFO,
+            'phoneid': PHONE_ID,
+            'access_token': self.access_token,
+            'signature2': signature
+        }
+
+        self._session.headers.update(headers)
+
+        req = self._session.prepare_request(requests.Request('POST', url, json=payload))
+
+        payload = json.dumps(payload, separators=(',', ':'))
+
+        req.body = payload.encode('utf-8')
+        req.prepare_content_length(req.body)
+
+        response_json = self._session.send(req).json()
+
+        self.check_for_errors_thermostat(response_json)
+
+        return response_json
+
+    @staticmethod
+    def check_for_errors_thermostat(response_json):
+        if response_json['code'] != 1:
+            raise UnknownApiError(response_json)
+
+    def get_plan_binding_list_by_user(self) -> Dict:
+        url = "https://wyze-membership-service.wyzecam.com/platform/v2/membership/get_plan_binding_list_by_user"
+        payload = olive_create_hms_payload()
+        signature = olive_create_signature(payload, self.access_token)
+        headers = {
+            'Accept-Encoding': 'gzip',
+            'User-Agent': 'myapp',
+            'appid': OLIVE_APP_ID,
+            'appinfo': APP_INFO,
+            'phoneid': PHONE_ID,
+            'access_token': self.access_token,
+            'signature2': signature
+        }
+
+        response_json = self._session.get(url, headers=headers, params=payload).json()
+
+        return response_json
+
+    def get_hms_id(self) -> Optional[str]:
+        response = self.get_plan_binding_list_by_user()
+        hms_subs = response['data']
+
+        if len(hms_subs) >= 1:
+            for sub in hms_subs:
+                if (devices := sub.get('deviceList')) is not None and len(devices) >= 1:
+                    for device in devices:
+                        return device['device_id']
+
+        return None
+
+    def monitoring_profile_active(self, hms_id: str, home: int, away: int) -> Dict:
+        url = "https://hms.api.wyze.com/api/v1/monitoring/v1/profile/active"
+        query = olive_create_hms_patch_payload(hms_id)
+        signature = olive_create_signature(query, self.access_token)
+        headers = {
+            'Accept-Encoding': 'gzip',
+            'User-Agent': 'myapp',
+            'appid': OLIVE_APP_ID,
+            'appinfo': APP_INFO,
+            'phoneid': PHONE_ID,
+            'access_token': self.access_token,
+            'signature2': signature,
+            'Authorization': self.access_token
+        }
+        payload = [
+            {
+                "state": "home",
+                "active": home
+            },
+            {
+                "state": "away",
+                "active": away
+            }
+        ]
+
+        response = self._session.patch(url, headers=headers, params=query, json=payload)
+
+        return response.json()
+
+    def monitoring_profile_state_status(self, hms_id):
+        url = "https://hms.api.wyze.com/api/v1/monitoring/v1/profile/state-status"
+        query = olive_create_hms_get_payload(hms_id)
+        signature = olive_create_signature(query, self.access_token)
+        headers = {
+            'User-Agent': 'myapp',
+            'appid': OLIVE_APP_ID,
+            'appinfo': APP_INFO,
+            'phoneid': PHONE_ID,
+            'access_token': self.access_token,
+            'signature2': signature,
+            'Authorization': self.access_token,
+            'Content-Type': "application/json"
+        }
+
+        response = self._session.get(url, headers=headers, params=query).json()
+
+        return response
